
/// Html tags.
library tags.html;

import 'dart:html';
export 'dart:html';
import 'builder.dart';

final Tag a         = ([Build b, _]) => build(() => new AnchorElement(), b);
final Tag area      = ([Build b, _]) => build(() => new AreaElement(), b);
final Tag b         = ([Build b_, _]) => build(() => _element('b'), b_);
final Tag base      = ([Build b, _]) => build(() => new BaseElement(), b);
final Tag body      = ([Build b, _]) => build(() => new BodyElement(), b);
final Tag br        = ([Build b, _]) => build(() => new BRElement(), b);
final Tag button    = ([Build b, _]) => build(() => new ButtonElement(), b);
final Tag canvas    = ([Build b, _]) => build(() => new CanvasElement(), b);
final Tag caption   = ([Build b,_]) => build(() => new TableCaptionElement(),b);
final Tag col       = ([Build b, _]) => build(() => new TableColElement(), b);
final Tag content   = ([Build b, _]) => build(() => new ContentElement(), b);
final Tag datalist  = ([Build b, _]) => build(() => new DataListElement(), b);
final Tag dd        = ([Build b, _]) => build(() => _element('dd'), b);
final Tag details   = ([Build b, _]) => build(() => new DetailsElement(), b);
final Tag div       = ([Build b, _]) => build(() => new DivElement(), b);
final Tag dl        = ([Build b, _]) => build(() => _element('dl'), b);
final Tag dt        = ([Build b, _]) => build(() => _element('dt'), b);
final Tag embed     = ([Build b, _]) => build(() => new EmbedElement(), b);
final Tag fieldset  = ([Build b, _]) => build(() => new FieldSetElement(), b);
final Tag form      = ([Build b, _]) => build(() => new FormElement(), b);
final Tag h1        = ([Build b, _]) => build(() => new HeadingElement.h1(), b);
final Tag h2        = ([Build b, _]) => build(() => new HeadingElement.h2(), b);
final Tag h3        = ([Build b, _]) => build(() => new HeadingElement.h3(), b);
final Tag h4        = ([Build b, _]) => build(() => new HeadingElement.h4(), b);
final Tag h5        = ([Build b, _]) => build(() => new HeadingElement.h5(), b);
final Tag h6        = ([Build b, _]) => build(() => new HeadingElement.h6(), b);
final Tag head      = ([Build b, _]) => build(() => new HeadElement(), b);
final Tag hr        = ([Build b, _]) => build(() => new HRElement(), b);
final Tag html      = ([Build b, _]) => build(() => new HtmlElement(), b);
final Tag i         = ([Build b, _]) => build(() => _element('i'), b);
final Tag img       = ([Build b, _]) => build(() => new ImageElement(), b);
final Tag input     = ([Build b, _]) => build(() => new InputElement(), b);
final Tag keygen    = ([Build b, _]) => build(() => new KeygenElement(), b);
final Tag label     = ([Build b, _]) => build(() => new LabelElement(), b);
final Tag legend    = ([Build b, _]) => build(() => new LegendElement(), b);
final Tag li        = ([Build b, _]) => build(() => new LIElement(), b);
final Tag link      = ([Build b, _]) => build(() => new LinkElement(), b);
final Tag map       = ([Build b, _]) => build(() => new MapElement(), b);
final Tag menu      = ([Build b, _]) => build(() => new MenuElement(), b);
final Tag meter     = ([Build b, _]) => build(() => new MeterElement(), b);
final Tag nav       = ([Build b, _]) => build(() => _element('nav'), b);
final Tag object    = ([Build b, _]) => build(() => new ObjectElement(), b);
final Tag ol        = ([Build b, _]) => build(() => new OListElement(), b);
final Tag optgroup  = ([Build b, _]) => build(() => new OptGroupElement(), b);
final Tag option    = ([Build b, _]) => build(() => new OptionElement(), b);
final Tag output    = ([Build b, _]) => build(() => new OutputElement(), b);
final Tag p         = ([Build b, _]) => build(() => new ParagraphElement(), b);
final Tag param     = ([Build b, _]) => build(() => new ParamElement(), b);
final Tag pre       = ([Build b, _]) => build(() => new PreElement(), b);
final Tag progress  = ([Build b, _]) => build(() => new ProgressElement(), b);
final Tag script    = ([Build b, _]) => build(() => new ScriptElement(), b);
final Tag select    = ([Build b, _]) => build(() => new SelectElement(), b);
final Tag source    = ([Build b, _]) => build(() => new SourceElement(), b);
final Tag span      = ([Build b, _]) => build(() => new SpanElement(), b);
final Tag style     = ([Build b, _]) => build(() => new StyleElement(), b);
final Tag table     = ([Build b, _]) => build(() => new TableElement(), b);
final Tag tbody     = ([Build b, _]) => build(() => _element('tbody'), b);
final Tag td        = ([Build b, _]) => build(() => new TableCellElement(), b);
final Tag template  = ([Build b, _]) => build(() => _element('template'), b);
final Tag textarea  = ([Build b, _]) => build(() => new TextAreaElement(), b);
final Tag tfoot     = ([Build b, _]) => build(() => _element('tfoot'), b);
final Tag th        = ([Build b, _]) => build(() => _element('th'), b);
final Tag thead     = ([Build b, _]) => build(() => _element('thead'), b);
final Tag title     = ([Build b, _]) => build(() => new TitleElement(), b);
final Tag tr        = ([Build b, _]) => build(() => new TableRowElement(), b);
final Tag track     = ([Build b, _]) => build(() => new TrackElement(), b);
final Tag u         = ([Build b, _]) => build(() => _element('u'), b);
final Tag ul        = ([Build b, _]) => build(() => new UListElement(), b);
final Tag video     = ([Build b, _]) => build(() => new VideoElement(), b);

/// Append the given [node] to the current [host].
///
/// An optional build function may be provided to build child elements.  This 
/// function provides a means to append existing elements in a tag builder 
/// expression.
Node append(Node node, [Build b]) => build(() => node, b);

/// Create a shadow root for the current [host].
///
/// This function may only be called from inside a [Build] function and the
/// host must be an element.
Node shadow([Build b]) {
  final _host = host;
  if (_host is! Element) {
    throw new StateError('shadow() may only be called if host is an Element.');
  }
  return build(() => _host.createShadowRoot(), b);
}

/// Append the given [text] as a text node to the current [host].
///
/// This function may only be called from inside a [Build] function and the
/// [host] must be an element.
void text(String text) {
  final _host = host;
  if (_host is! Element) 
    throw new StateError('text() may only be called if host is an Element.');
  _host.appendText(text);
}

// Create an Element for a `tag` that is untyped in `dart:html`.  E.g. `<th>`
Element _element(String tag) {
  if (!Element.isTagSupported(tag)) throw new UnsupportedError(tag);
  return new Element.tag(tag);
}
