
part of tags.custom;

/// A stateful button that allows toggling between an [up] and a [down] state.
class ToggleButton implements HasElement {
  
  final Element element = 
      button(() {
        shadow(() {
          content()..select='.up';  
        });
      });
  
  ContentElement _content;  
  
  bool _down = false;
  /// Whether or not this button is down.
  bool get down => _down;
       set down(bool value) {
         if (value != _down) {
           _down = value;           
           if (_down) _content.select = '.down';
           else _content.select = '.up';
           if (_states.hasListener && !_states.isPaused) _states.add(_down);
         }
       }
  
  /// Whether or not this button is up.
  bool get up => !_down;
  
  final _states = new StreamController<bool>();
  /// An observable stream of this button's state transitions.
  /// A _true_ value represents [down] and a _false_ value represents [up].
  Stream<bool> get states => _states.stream;
  
  /// Constructs a new toggle button.  
  ToggleButton() {
    element.onClick.listen((_) => down = !_down);
    _content = element.shadowRoot.children[0];
  }
}
